#! /usr/bin/perl
# setup.pl - This script is run to configure the App in the Portal.
# Created by James Pattie, 07/02/2001.  Copyright (c) 2000 PC & Web Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

use strict;

my $configVersion = "0.1";  # Current Configuration File Version to be working with.
my $appName = "squidGuard";   # Change these values to reflect the "App" being configured.
my $dbName = "squidguard_db";  # specify the name of your database if you need one.
my $dbType = "Pg";  # either Pg for PostgreSQL or mysql for MySQL.
my $dbPerCompany = "false";  # true if you need a database per company.
my $appType = "tools";  # one of administration, help, bugs, regular, tools or news
my $appAdminType = "normal";  # If appType = administration, then you can have a 'normal' or 'system' admin app.
my $appDescription = "squidGuard Database Administration App";
my $appIconName = "squidGuard.gif";
my $helpDefined = "true";  # specify false if no Help.pm module created.

# default values to use.
my $isAppFree = "true";  # Set to false if app is not free.
my $appCost = "0.00";  # Set to price if app is not free.
my $unit = 0;  # 0 = per month, 1 = per user.
my $numLicenses = 0;  # 0 means unlimited.  Can only be changed if app is not free.  Is only used if assigning to all companies.

my $wapAware = "false";  # Is this app Wap Aware?

print "\n#################  Portal::$appName Configuration settings #################\n";

my $configFile = "/etc/pcx_portal/$appName/Config.pm";

# make sure the config file exists.
if (! -e $configFile)
{
  die "Error:  Config file = '$configFile' does not exist!\n";
}

# pull in the settings from the config file.
my $return = "";
unless ($return = do $configFile)
{
  die "Couldn't parse '$configFile': $@<br>\n" if $@;
  die "Couldn't do '$configFile': $!<br>\n" unless defined $return;
  die "Couldn't run '$configFile'" unless $return;
}
my %settings = ();
%settings = %PortalsquidGuardSettings::settings;       # <------- Change App to Whatever your App name is!  ------------------

RESTART:  # We go here if they say No to use current settings.
my $useCurrent = 0;
my $input = "";
my %configItems = %settings;  # Make sure we get all values that were there before so that they get put back out if we don't modify them.

#if ($configItems{configVersion} eq "0.1")
#{
  # This is where you could put in anything that is new that you won't be asking
  # the user for the value of (color constants, etc.)
  #$configItems{configVersion} = "0.2";
#}

$configItems{appName} = $appName;  # make sure we have the correct AppName in the config file.
$configItems{appType} = $appType;
$configItems{appAdminType} = $appAdminType;
$configItems{appDescription} = $appDescription;
$configItems{appIconName} = $appIconName;
$configItems{dbType} = $dbType;
$configItems{dbPerCompany} = $dbPerCompany;
$configItems{helpDefined} = $helpDefined;

# ****************************************************

print "\nDatabase Type = '$configItems{dbType}'\n";

# get the Database Port
$configItems{dbPort} = getInput(prompt => "Enter Database Server Port", value => ($settings{dbPort} ne "DBPORT" ? $settings{dbPort} : ($configItems{dbType} eq "Pg" ? "5432" : "3306")), mask => "\\d+");

print "\nDatabase Per Company = $dbPerCompany\n";
# get the Database Name
if ($configItems{dbPerCompany} eq "true")
{
  $configItems{dbName} = getInput(prompt => "Enter Database Name to append to the Company Name", value => ($settings{dbName} ne "DBNAME" ? $settings{dbName} : "$dbName"));
}
else
{
  $configItems{dbName} = $dbName;
}

$configItems{width} = getInput(prompt => "Default Width of App Window", value => ($settings{width} ne "WIDTH" ? $settings{width} : "640"), mask => "\\d+");
$configItems{height} = getInput(prompt => "Default Height of App Window", value => ($settings{height} ne "HEIGHT" ? $settings{height} : "480"), mask => "\\d+");
$configItems{autorun} = getInput(prompt => "Should App default to automatically start when a user logs in?  (true or false)", value => ($settings{autorun} ne "AUTORUN" ? $settings{autorun} : "false"), mask => "(true|false)");

print "\nAppType = '$appType'\n";
print "Administration Type = '$appAdminType'\n" if ($appType eq "administration");

$configItems{isAppFree} = getInput(prompt => "Is this App Free? (true or false)", value => ($settings{isAppFree} ne "ISAPPFREE" ? $settings{isAppFree} : $isAppFree), mask => "(true|false)");
if ($configItems{isAppFree} eq "false")
{
  $configItems{appCost} = getInput(prompt => "Cost of App", value => ($settings{appCost} ne "APPCOST" ? $settings{appCost} : $appCost), mask => "\d+\.\d{2}");
  $configItems{unit} = getInput(prompt => "Per Month (0) or Per User (1)", value => ($settings{unit} ne "UNIT" ? $settings{unit} : $unit), mask => "(0|1)");
  $configItems{numLicenses} = getInput(prompt => "Number of licenses to defaultly assign", value => ($settings{numLicenses} ne "NUMLICENSES" ? $settings{numLicenses} : $numLicenses), mask => "\d+");
}
else
{
  $configItems{appCost} = "0.00";
  $configItems{unit} = 0;
  $configItems{numLicenses} = 0;
}

$configItems{assignToAllCompanies} = getInput(prompt => "Assign to All Companies (true or false) (If false, then no assignment to users will be done)", value => ($settings{assignToAllCompanies} ne "ASSIGNTOALLCOMPANIES" ? $settings{assignToAllCompanies} : "true"), mask => "(true|false)");
if ($configItems{assignToAllCompanies} eq "true")
{
  if ($appType eq "administration")
  {
    $configItems{assignToAllUsers} = "false";  # This isn't possible with an administration app.
    if ($appAdminType eq "normal")
    {
      $configItems{assignToAdminUsers} = getInput(prompt => "Assign to All Admin Users (true or false)", value => ($settings{assignToAdminUsers} ne "ASSIGNTOADMINUSERS" ? $settings{assignToAdminUsers} : "true"), mask => "(true|false)");
      if ($configItems{assignToAdminUsers} eq "false")
      {
        $configItems{assignToSysAdminUsers} = getInput(prompt => "Assign Only to System Admin Users (true or false)", value => ($settings{assignToSysAdminUsers} ne "ASSIGNTOSYSADMINUSERS" ? $settings{assignToSysAdminUsers} : "true"), mask => "(true|false)");
      }
      else
      {
        $configItems{assignToSysAdminUsers} = "false";  # don't need this set as the AdminUsers also covers SysAdmin users.
      }
    }
    else
    {
      $configItems{assignToAdminUsers} = "false";  # This can't be done with a SysAdmin App.
      $configItems{assignToSysAdminUsers} = getInput(prompt => "Assign to System Admin Users (true or false)", value => ($settings{assignToSysAdminUsers} ne "ASSIGNTOSYSADMINUSERS" ? $settings{assignToSysAdminUsers} : "true"), mask => "(true|false)");
    }
  }
  else
  {
    $configItems{assignToAllUsers} = getInput(prompt => "Assign To All Users (true or false)", value => ($settings{assignToAllUsers} ne "ASSIGNTOALLUSERS" ? $settings{assignToAllUsers} : "true"), mask => "(true|false)");
    if ($configItems{assignToAllUsers} eq "false")
    {
      $configItems{assignToAdminUsers} = getInput(prompt => "Assign to All Admin Users (true or false)", value => ($settings{assignToAdminUsers} ne "ASSIGNTOADMINUSERS" ? $settings{assignToAdminUsers} : "true"), mask => "(true|false)");
      if ($configItems{assignToAdminUsers} eq "false")
      {
        $configItems{assignToSysAdminUsers} = getInput(prompt => "Assign Only to System Admin Users (true or false)", value => ($settings{assignToSysAdminUsers} ne "ASSIGNTOSYSADMINUSERS" ? $settings{assignToSysAdminUsers} : "true"), mask => "(true|false)");
      }
      else
      {
        $configItems{assignToSysAdminUsers} = "false";  # don't need this set as the AdminUsers also covers SysAdmin users.
      }
    }
    else
    { # no need to set the Admin or SysAdmin flags as they are automatically included.
      $configItems{assignToAdminUsers} = "false";
      $configItems{assignToSysAdminUsers} = "false";
    }
  }
}
else
{
  $configItems{assignToAllUsers} = "false";
  $configItems{assignToAdminUsers} = "false";
  $configItems{assignToSysAdminUsers} = "false";
}

$configItems{wapAware} = $wapAware;

# ****************************************************

# verify they want to use these settings.
$input = getInput(prompt => "Accept these values (Y) or Redo (N)?", value => "Y", mask => "Y|N|y|n");
if ($input =~ /^N$/i)
{
  goto RESTART;
}

print "Generating Config file '$configFile'...\n";

# generate the Config file now.
open (F, ">$configFile") or die "Error:  Couldn't open '$configFile'!  Error = $!\n";

my $package = "Portal" . $appName . "Settings";
my $date = `/bin/date`;
chomp $date;

print F <<"END_OF_FILE";
# Config.pm - The config file for the Portal::$appName.
# Created by James Pattie (james\@pcxperience.com)
# $date - Copyright (c) 2001, PC & Web Xperience, Inc.
# (http://www.pcxperience.com/)

package $package;

use vars '%settings';

%settings = ();

\$settings{configVersion} = "$configVersion";
END_OF_FILE

foreach my $name (keys %configItems)
{
  if ($name ne "configVersion")
  {
    print F "\$settings{$name} = \"$configItems{$name}\";\n";
  }
}

print F "\n1;\n";

close (F)		or die "can't close $configFile: $!";

print "\nDon't forget to run appinstall.pl to actually register this app with the Portal.\n";

# getInput - prompt, mask, and value
sub getInput
{
  my %args = (prompt => "", mask => ".*", value => "", @_);
  my $prompt = $args{prompt};
  my $mask = $args{mask};
  my $value = $args{value};

  my $input = "";
  my $done = 0;
  while (!$done)
  {
    print "\n$prompt [$value]: ";
    $input = <STDIN>;
    chomp $input;
    if (length $input == 0)
    {
      $input = $value;
    }
    if ($input !~ /^($mask)$/)
    {
      print "'$input' is invalid!\n";
      $input = "";
    }
    else
    {
      $done = 1;  # This will let me out even if I'm entering an empty string, as long as that is valid.
    }
  }
  $input =~ s/@/\\@/g;  # make sure that any @'s are escaped.
  $input =~ s/%/\\%/g;  # make sure that any %'s are escaped.
  return $input;
}

1;
