# Main.pm - The Object Class that provides a Main Object
# Created by James A. Pattie, 10/30/2001.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Main;
use strict;
use Portal::squidGuard::Data::Config;
use Portal::Data::Variables;
use Portal::Data::Config;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( display => "Main Menu", close => "Close App" );

# new
# instantiates an instance of the Main State
# takes: langObj, input, cookieObj, dbHandle, logObj, sessionObj, userObj, companyObj, applicationObj, appObj, portalSession, browserType, browserVersion, browserMaker, authObj
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( langObj => undef, input => undef, cookieObj => undef, dbHandle => undef, logObj => undef, sessionObj => undef, userObj => undef, companyObj => undef,
               applicationObj => undef, appObj => undef, portalSession => undef, browserType => "HTML", browserVersion => "4.0", browserMaker => "", authObj => undef, @_ );
  my $errStr = "Portal::squidGuard::Main->new() - Error!<br>\n";

  $self->{portalVariables} = Portal::Data::Variables->new;
  my $configObj = undef;
  eval { $configObj = Portal::squidGuard::Data::Config->new; };
  if ($@)
  {
    $self->setError(errorString => $errStr . "eval of Portal::squidGuard::Data::Config->new failed!<br>\nError = '$@'<br>\n");
    return $self;
  }
  $self->{configObj} = $configObj;
  $self->{langObj} = $args{langObj};
  $self->{input} = $args{input};
  $self->{cookieObj} = $args{cookieObj};
  $self->{dbHandle} = $args{dbHandle};
  $self->{logObj} = $args{logObj};
  $self->{sessionObj} = $args{sessionObj};
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{applicationObj} = $args{applicationObj};
  $self->{appObj} = $args{appObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Main->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{langObj})
  {
    $errorString .= "langObj must be defined!<br>\n";
  }
  elsif (!$self->{langObj}->isValid)
  {
    $errorString .= $self->{langObj}->errorMessage;
  }
  if (not defined $self->{applicationObj})
  {
    $errorString .= "applicationObj must be defined!<br>\n";
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $errorString .= $self->{applicationObj}->errorMessage;
  }
  if (not defined $self->{appObj})
  {
    $errorString .= "appObj must be defined!<br>\n";
  }
  elsif (!$self->{appObj}->isValid)
  {
    $errorString .= $self->{appObj}->errorMessage;
  }
  if (not defined $self->{portalSession})
  {
    $errorString .= "portalSession must be defined!<br>\n";
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $errorString .= $self->{portalSession}->errorMessage;
  }
  if (not defined $self->{userObj})
  {
    $errorString .= "userObj must be defined!<br>\n";
  }
  elsif (!$self->{userObj}->isValid)
  {
    $errorString .= $self->{userObj}->errorMessage;
  }
  if (not defined $self->{companyObj})
  {
    $errorString .= "companyObj must be defined!<br>\n";
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $errorString .= $self->{companyObj}->errorMessage;
  }
  if (not defined $self->{authObj})
  {
    $errorString .= "authObj must be defined!<br>\n";
  }
  elsif (!$self->{authObj}->isValid)
  {
    $errorString .= $self->{authObj}->errorMessage;
  }
  if (not defined $self->{logObj})
  {
    $errorString .= "logObj must be defined!<br>\n";
  }
  elsif (!$self->{logObj}->isValid)
  {
    $errorString .= $self->{logObj}->errorMessage;
  }
  if (not defined $self->{cookieObj})
  {
    $errorString .= "cookieObj must be defined!<br>\n";
  }
  if (not defined $self->{dbHandle})
  {
    $errorString .= "dbHandle must be defined!<br>\n";
  }
  if (not defined $self->{sessionObj})
  {
    $errorString .= "sessionObj must be defined!<br>\n";
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# run
# takes: command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Main->run()  - $errorPhrase<br>\n";

  if (!exists $commands{$command})
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{commandNotKnown}, $command));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    my $bgColor = $self->{configObj}->{bgColor};
    my $fgColor = $self->{configObj}->{fgColor};
    my $menufgColor = $self->{configObj}->{menufgColor};
    my $menubgColor = $self->{configObj}->{menubgColor};
    my $cellfgColor = $self->{configObj}->{cellfgColor};
    my $cellbgColor = $self->{configObj}->{cellbgColor};
    my $cellTitlefgColor = $self->{configObj}->{cellTitlefgColor};
    my $cellTitlebgColor = $self->{configObj}->{cellTitlebgColor};
    my $tablebgColor = $self->{configObj}->{tablebgColor};

    my $close = $phrases->{close};
    my $refresh = $phrases->{refresh};
    my $help = $phrases->{help};
    my $mainMenu = $phrases->{mainMenu};
    my $title = sprintf($phrases->{title}, $Portal::squidGuard::VERSION, $mainMenu);
    my $closeMessage = $phrases->{closeMessage};
    my $domainAndUrlPhrase = $phrases->{domainAndUrl};
    my $settingsPhrase = $phrases->{settings};
    my $serverSettingsPhrase = $phrases->{serverSettings};
    my $exportPhrase = $phrases->{export};
    my $noServersCreatedPhrase = $phrases->{noServersCreated};
    my $gotoSettingsPhrase = $phrases->{gotoSettings};
    my $selectPhrase = $phrases->{select};
    my $allowedPhrase = $phrases->{allowed};
    my $blackListsPhrase = $phrases->{blackLists};
    my $noGroupsSelectedPhrase = $phrases->{noGroupsSelected};
    my $serverInvalidPhrase = $phrases->{serverInvalid};
    my $selectServerToWorkWithPhrase = $phrases->{selectServerToWorkWith};
    my $serverPhrase = $phrases->{server};
    my $exportMessage = $phrases->{exportMessage};

    my $server = $self->{input}->{server};

    $doc->setTitle($title);

    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");

function showHelp(appName, url, height, width)
{
  var w = window.open(url, appName, "height="+height+",width="+width+",resizable,scrollbars,status");
}

function validateClose()
{
  return window.confirm("$closeMessage");
}

function validateExport()
{
  return window.confirm("$exportMessage");
}

END_OF_CODE
    $doc->setLinkDecorations();  # turn off underlines on the links.

    my $configObj = Portal::Data::Config->new;
    my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";
    my $closeUrl = $url . "?app=squidGuard&state=Main&command=close";
    my $refreshUrl = $url . "?app=squidGuard&state=Main&command=display";
    my $helpUrl = $url . "?app=Portal&state=Help&command=display&helpApp=squidGuard&content=Normal";
    my $userUrl = $url . "?app=squidGuard&state=Server&command=display";
    my $safeServer = $doc->encodeString(string => $server);

    # first verify they have some servers configured
    my @servers = ();
    my %servers = ();
    my $sth = $self->{dbHandle}->read(sql => "SELECT server, id FROM servers_tb WHERE cid = ? ORDER BY server", plug => [ $self->{companyObj}->{id} ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }
    while (my @info = $sth->fetchrow_array)
    {
      $servers[++$#servers] = $info[0];
      $servers{$info[0]} = $info[1];
    }
    $sth->finish;

    if (scalar @servers > 0 && exists $servers{$server})
    {
      $refreshUrl .= "&server=$safeServer";
    }

    $doc->setFocus("body");
    $doc->print(<<"END_OF_CODE");
<center>
<table border="0" width="100%" cellspacing="0" cellpadding="2" bgcolor="$menubgColor">
  <tr>
    <td align="left" width="100%"><a href="$closeUrl" onMouseOver="window.status='$close'; return true;" title="$close" onClick="return validateClose();"><font color="$menufgColor">$close</font></a> &nbsp;| &nbsp;
    <a href="$userUrl" onMouseOver="window.status='$serverSettingsPhrase'; return true;" title="$serverSettingsPhrase"><font color="$menufgColor">$settingsPhrase</font></a> &nbsp; | &nbsp;
    <a href="$helpUrl" onMouseOver="window.status='$help'; return true;" title="$help" onClick="showHelp('help_squidGuard', '$helpUrl', 480, 640); return false;"><font color="$menufgColor">$help</font></a> &nbsp;| &nbsp;
    <a href="$refreshUrl" onMouseOver="window.status='$refresh'; return true;" title="$refresh"><font color="$menufgColor">$refresh</font></a>
END_OF_CODE

    if (scalar @servers > 0 && exists $servers{$server})
    {
      my $exportUrl = $url . "?app=squidGuard&state=Rules&command=export&serverID=" . $servers{$server};
      # lookup the version we are at.
      $sth = $self->{dbHandle}->read(sql => "SELECT version FROM servers_tb where id = ?", plug => [ $servers{$server} ]);
      if ($self->{dbHandle}->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
        return undef;
      }
      my @info = $sth->fetchrow_array;
      $sth->finish;
      my $version = $info[0];

      $doc->print(<<"END_OF_CODE");
    &nbsp;| &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <a href="$exportUrl" onMouseOver="window.status='$exportPhrase'; return true;" title="$exportPhrase" onClick="return validateExport();"><font color="$menufgColor">$exportPhrase (Version = <b>$version</b>)</font></a>
END_OF_CODE
    }
    $doc->print(<<"END_OF_CODE");
    </td>
  </tr>
</table>
END_OF_CODE

    if (scalar @servers == 0)
    {
      $doc->print(<<"END_OF_CODE");
<br>
<h1>$noServersCreatedPhrase</h1>
$gotoSettingsPhrase
END_OF_CODE
    }
    else
    {
      $doc->print(<<"END_OF_CODE");
<br>
<form name="selectServer" method="post" action="$url">
  <input type="hidden" name="app" value="squidGuard">
  <input type="hidden" name="state" value="Main">
  <input type="hidden" name="command" value="display">
  $serverPhrase: <select name="server">
END_OF_CODE
      foreach my $entry (@servers)
      {
        my $output = "    <option value=\"$entry\"" . ($entry eq $server ? " selected" : "") . ">$entry\n";
        $doc->print($output);
      }
      $doc->print(<<"END_OF_CODE");
  </select> <input type="submit" name="go" value="$selectPhrase">
</form>
END_OF_CODE

      if (length $server > 0 && exists $servers{$server})
      {
        # figure out how many domains and urls entries are in the allowed table.
        my %allowedTotals = ();
        foreach my $name (qw(domain url))
        {
          my $table = "allowed_$name" . "s_tb";
          my $sth = $self->{dbHandle}->read(sql => "SELECT COUNT($name) FROM $table WHERE sid = ?", plug => [ $servers{$server} ]);
          if ($self->{dbHandle}->didErrorOccur)
          {
            $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
            return undef;
          }
          my @info = $sth->fetchrow_array;
          $allowedTotals{$name} = $info[0];
        }
        my $allowedUrl = $url . "?app=squidGuard&state=Rules&command=display&type=allowed&serverID=$servers{$server}";

        $doc->print(<<"END_OF_CODE");
<br>
<table border="0" cellspacing="0" cellpadding="2" bgColor="$tablebgColor">
  <tr bgcolor="$cellTitlebgColor">
    <th colspan="2" align="center"><font color="$cellTitlefgColor">$allowedPhrase</font></th>
  </tr>
  <tr>
    <td align="left"><a href="$allowedUrl" onMouseOver="window.status='$domainAndUrlPhrase'; return true;" title="$domainAndUrlPhrase">$domainAndUrlPhrase</a></td>
    <td align="right">($allowedTotals{domain} | $allowedTotals{url})</td>
  </tr>
  <tr bgcolor="$cellTitlebgColor">
    <th colspan="2" align="center"><font color="$cellTitlefgColor">$blackListsPhrase</font></th>
  </tr>
END_OF_CODE

        # display the links that are available to the user so they can work in the different sections.

        # figure out what Black List groups the user is working with and display their links.
        my $sth = $self->{dbHandle}->read(sql => "SELECT bl_groups FROM servers_tb WHERE id = ?", plug => [ $servers{$server} ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        my @info = $sth->fetchrow_array;
        if (!defined $info[0])
        {
          $doc->print(<<"END_OF_CODE");
  <tr>
    <td colspan="2" align="center">$noGroupsSelectedPhrase</td>
  </tr>
END_OF_CODE
        }
        my $selectedBlackLists = $info[0];
        my @selectedBlackLists = split (/\|/, $selectedBlackLists);

        foreach my $entry (sort @selectedBlackLists)
        {
          my $safeEntry = $doc->encodeString(string => $entry);
          my $blUrl = $url . "?app=squidGuard&state=Rules&command=display&type=bl&group=$safeEntry&serverID=$servers{$server}";

          # figure out how many domains and urls entries are in the blacklist table.
          my %blTotals = ();
          foreach my $name (qw(domain url))
          {
            my $table = "bl_$name" . "s_tb";
            my $sth = $self->{dbHandle}->read(sql => "SELECT COUNT($name) FROM $table WHERE sid = ? AND bl_group = ?", plug => [ $servers{$server}, $entry ]);
            if ($self->{dbHandle}->didErrorOccur)
            {
              $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
              return undef;
            }
            my @info = $sth->fetchrow_array;
            $blTotals{$name} = $info[0];
          }

          $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="left"><a href="$blUrl" onMouseOver="window.status='$entry - $domainAndUrlPhrase'; return true;" title="$entry - $domainAndUrlPhrase">$entry</a></td>
    <td align="right">($blTotals{domain} | $blTotals{url})</td>
  </tr>
END_OF_CODE
        }
        $doc->print(<<"END_OF_CODE");
</table>
</center>
<br>
<ul>
  <li><b>Allowed</b> allows you to specify <b>domains</b> and <b>urls</b> entries that should be allowed (because they are
currently denied or because you are in a allow only scenario).</li>
  <li><b>Black Lists</b> group the <b>domains</b> or <b>urls</b> entries that should be blocked. &nbsp;These are pre-determined categorizations and can not be
added to. &nbsp;You can select which categories show up though from the <b>Settings</b> screen for this server.</li>
  <li>To the right of the Allowed entry or each group in the Black Lists section is a (# | #) where the first number is the number of domains entries and the
second number is the number of urls entries.</li>
  <li>Only <b>Export</b> the changes you have made (which just increments a version counter) when you are done making changes.  This will prevent your firewall from constantly
pulling down a new version of the config file when you aren't fully done making changes.  Remember that the firewall is configured to only query about every 5 minutes
and so your changes may not go into affect for at max 10 minutes (based upon the time to download the changes and restart the squid/squidGuard services on the firewall).</li>
</ul>
END_OF_CODE
      }
      else
      {
        if (length $server > 0)
        {
          $doc->print("<br><h1>" . sprintf($serverInvalidPhrase, $server) . "</h1><br>\n");
        }
        $doc->print("<br><h2>$selectServerToWorkWithPhrase</h2>\n");
      }
    }
  }
  elsif ($command eq "close")
  {
    if (!$self->sessionObj->{store}->{okToLogout})
    {
      $doc = $self->run(command => "display");
      if ($self->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->errorMessage);
        return undef;
      }
    }
    else
    {
      my $close = $phrases->{close};
      my $title = sprintf($phrases->{title}, $VERSION, $close);

      # generate the document that will close all open windows and then close itself.
      $doc->setTitle($title);

      $doc->setFocus("javascript");
      $doc->print(<<"END_OF_CODE");

function closeWindow(wname)
{
  var w = open("", wname);
  if (!w.closed)
  {
    w.close();
  }
}

END_OF_CODE

      my $onLoadCode = "";
      foreach my $window (keys %{$self->sessionObj->{store}->{windows}})
      {
        $onLoadCode .= "closeWindow('$window');\n";
      }

      my $windowName = "help_squidGuard";
      if (exists $self->{portalSession}->{store}->{windows}->{$windowName})
      {
        $onLoadCode .= "closeWindow('$windowName');\n";
        delete $self->{portalSession}->{store}->{windows}->{$windowName};
        $self->{portalSession}->{store}->{changed} = 1;
      }

      $onLoadCode .= "\nwindow.close();\n";  # close ourself

      if (length $onLoadCode > 0)
      {
        $doc->setOnload(code => "$onLoadCode");  # output the JavaScript code that will be run on the onLoad event of the page.
      }

      $self->{sessionObj}->delete;
      if ($self->sessionObj->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->sessionObj->errorMessage);
        return undef;
      }
    }
  }

  return $doc;
}

1;
__END__

=head1 NAME

Main - Object used to build a Main Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Main;
  my $obj = Portal::squidGuard::Main->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is a Main class.

=head1 Exported FUNCTIONS

  scalar new(langObj, input, cookieObj, dbHandle, logObj, sessionObj,
             userObj, companyObj, applicationObj, appObj, portalSession,
             browserType, browserVersion, browserMaker, authObj)
    Creates a new instance of the Portal::Main
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
