# PortalSetup.pm - The Object Class that provides a PortalSetup Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::PortalSetup;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( cleanupDatabase => "Cleans up the Database", setupCompany => "Sets up a Company in the app", setupUser => "Sets up a User in the app", updateUser => "Updates a User in the app",
                 cleanupUser => "Removes a User from the App", );

# new
# instantiates an instance of a PortalSetup
# takes: langObj, input, cookieObj, dbHandle, logObj, sessionObj, userObj, companyObj, browserType, browserVersion, browserMaker, authObj
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( langObj => undef, input => undef, cookieObj => undef, dbHandle => undef, logObj => undef, sessionObj => undef, userObj => undef, companyObj => undef,
               applicationObj => undef, appObj => undef, portalSession => undef, browserType => "HTML", browserVersion => "4.0", browserMaker => "", authObj => undef, @_ );
  my $errStr = "Portal::squidGuard::PortalSetup->new() - Error!<br>\n";

  $self->{langObj} = $args{langObj};
  $self->{input} = $args{input};
  $self->{cookieObj} = $args{cookieObj};
  $self->{dbHandle} = $args{dbHandle};
  $self->{logObj} = $args{logObj};
  $self->{sessionObj} = $args{sessionObj};
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{applicationObj} = $args{applicationObj};
  $self->{appObj} = $args{appObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::PortalSetup->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{langObj})
  {
    $errorString .= "langObj must be defined!<br>\n";
  }
  elsif (!$self->{langObj}->isValid)
  {
    $errorString .= $self->{langObj}->errorMessage;
  }
  if (not defined $self->{applicationObj})
  {
    $errorString .= "applicationObj must be defined!<br>\n";
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $errorString .= $self->{applicationObj}->errorMessage;
  }
  if (not defined $self->{appObj})
  {
    $errorString .= "appObj must be defined!<br>\n";
  }
  elsif (!$self->{appObj}->isValid)
  {
    $errorString .= $self->{appObj}->errorMessage;
  }
  if (not defined $self->{portalSession})
  {
    $errorString .= "portalSession must be defined!<br>\n";
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $errorString .= $self->{portalSession}->errorMessage;
  }
  if (not defined $self->{userObj})
  {
    $errorString .= "userObj must be defined!<br>\n";
  }
  elsif (!$self->{userObj}->isValid)
  {
    $errorString .= $self->{userObj}->errorMessage;
  }
  if (not defined $self->{companyObj})
  {
    $errorString .= "companyObj must be defined!<br>\n";
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $errorString .= $self->{companyObj}->errorMessage;
  }
  if (not defined $self->{authObj})
  {
    $errorString .= "authObj must be defined!<br>\n";
  }
  elsif (!$self->{authObj}->isValid)
  {
    $errorString .= $self->{authObj}->errorMessage;
  }
  if (not defined $self->{logObj})
  {
    $errorString .= "logObj must be defined!<br>\n";
  }
  elsif (!$self->{logObj}->isValid)
  {
    $errorString .= $self->{logObj}->errorMessage;
  }
  if (not defined $self->{cookieObj})
  {
    $errorString .= "cookieObj must be defined!<br>\n";
  }
  if (not defined $self->{dbHandle})
  {
    $errorString .= "dbHandle must be defined!<br>\n";
  }
  if (not defined $self->{sessionObj})
  {
    $errorString .= "sessionObj must be defined!<br>\n";
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# run
# takes: command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $phrases = $self->langObj->phraseObj->phrases;
  my $errStr = "Portal::squidGuard::PortalSetup->run()  - Error!<br>\n";

  if (!exists $commands{$command})
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{commandNotKnown}, $command));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "cleanupDatabase")
  {
    $self->cleanupDatabase(dataDir => $self->{input}->{dataDir});
    if ($self->didErrorOccur)
    {
      $doc->setFocus("body");
      $doc->print("Cleanup of Database failed!...<br>\nError = " . $self->errorMessage);

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 0;");
    }

    $doc->setFocus("body");
    $doc->print("Cleanup Of Database Succeeded...");

    $doc->setOnload(code => "window.opener.configForm.configResult.value = 1;\nwindow.close();");
  }
  elsif ($command eq "setupCompany")
  {
    $doc = $self->setupCompany();
    if ($self->didErrorOccur)
    {
      $doc = HTMLObject::Normal->new;
      $doc->setFocus("body");
      $doc->print("Setup of Company failed!...<br>\nError = " . $self->errorMessage);

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 0;");
    }

    if (!defined $doc)
    {
      $doc = HTMLObject::Normal->new;

      $doc->setFocus("body");
      $doc->print("Setup Of Company Succeeded...");

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 1;\nwindow.close();");
    }
  }
  elsif ($command eq "setupUser")
  {
    $doc = $self->setupUser();
    if ($self->didErrorOccur)
    {
      $doc = HTMLObject::Normal->new;
      $doc->setFocus("body");
      $doc->print("Setup of User failed!...<br>\nError = " . $self->errorMessage);

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 0;");
    }

    if (!defined $doc)
    {
      $doc = HTMLObject::Normal->new;

      $doc->setFocus("body");
      $doc->print("Setup Of User Succeeded...");

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 1;\nwindow.close();");
    }
  }
  elsif ($command eq "updateUser")
  {
    $doc = $self->updateUser();
    if ($self->didErrorOccur)
    {
      $doc = HTMLObject::Normal->new;
      $doc->setFocus("body");
      $doc->print("Update of User failed!...<br>\nError = " . $self->errorMessage);

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 0;");
    }

    if (!defined $doc)
    {
      $doc = HTMLObject::Normal->new;

      $doc->setFocus("body");
      $doc->print("Update Of User Succeeded...");

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 1;\nwindow.close();");
    }
  }
  elsif ($command eq "cleanupUser")
  {
    $self->cleanupUser();
    if ($self->didErrorOccur)
    {
      $doc->setFocus("body");
      $doc->print("Cleanup of User failed!...<br>\nError = " . $self->errorMessage);

      $doc->setOnload(code => "window.opener.configForm.configResult.value = 0;");
    }

    $doc->setFocus("body");
    $doc->print("Cleanup Of User Succeeded...");

    $doc->setOnload(code => "window.opener.configForm.configResult.value = 1;\nwindow.close();");
  }

  return $doc;
}

# cleanupDatabase
# takes: dataDir
# returns: nothing
# This routine dumps any data necessary and if a database is defined, destroys the database.
sub cleanupDatabase
{
  my $self = shift;
  my $companyObj = $self->{companyObj};
  my $userObj = $self->{userObj};
  my $appObj = $self->{appObj};
  my %args = ( @_ );
  my $dataDir = $args{dataDir};
  my $errStr = "Portal::squidGuard->cleanupDatabase()  - Error!<br>\n";

  if (length $dataDir == 0 || ! -d $dataDir)
  {
    $self->setError(errorString => $errStr . "dataDir = '$dataDir' is invalid or does not exist!<br>\n");
    return;
  }

  # do any cleanup necessary
}

# setupCompany
# takes: nothing
# returns: doc
# This routine proceeds to create a database for the company if needed and does any initialization necessary.  The application needs to provide an HTMLObject document that will start the user going through a configuration
# program for that application.  If the doc is undef on return and no error is set, then the calling app will assume that no extra configuration needs to be done.
# The calling program should have this output being done in another window and provide the user with a link to close that window when all configuration is done.
sub setupCompany
{
  my $self = shift;
  my $companyObj = $self->{companyObj};
  my $userObj = $self->{userObj};
  my $appObj = $self->{appObj};
  my $doc = undef;
  my $errStr = "Portal::squidGuard->setupCompany()  - Error!<br>\n";

  # do any setup necessary

  return $doc;
}

# setupUser
# takes: nothing
# returns: doc
# This routine configures the user in the application.  The application needs to provide an HTMLObject document that will start the user going through a configuration
# program for that application.  If the doc is undef on return and no error is set, then the calling app will assume that no extra configuration needs to be done.
# The calling program should have this output being done in another window and provide the user with a link to close that window when all configuration is done.
sub setupUser
{
  my $self = shift;
  my $companyObj = $self->{companyObj};
  my $userObj = $self->{userObj};
  my $appObj = $self->{appObj};
  my $doc = undef;
  my $errStr = "Portal::squidGuard->setupUser()  - Error!<br>\n";

  # do any setup necessary

  return $doc;
}

# updateUser
# takes: nothing
# returns: doc
# This routine updates the users settings in the application.  This is mainly to synchronize any changes that were made to the users info in the portal_db
# that the application duplicates or relies on for user settings in its own database.  The application needs to provide an HTMLObject document that will start the user going through a configuration
# program for that application.  If the doc is undef on return and no error is set, then the calling app will assume that no extra configuration needs to be done.
# The calling program should have this output being done in another window and provide the user with a link to close that window when all configuration is done.
sub updateUser
{
  my $self = shift;
  my $companyObj = $self->{companyObj};
  my $userObj = $self->{userObj};
  my $appObj = $self->{appObj};
  my $doc = undef;
  my $errStr = "Portal::squidGuard->updateUser()  - Error!<br>\n";

  # do any setup necessary

  return $doc;
}

# cleanupUser
# takes: nothing
# returns: nothing
# This routine archives any info on the user and then removes their data from active use.
sub cleanupUser
{
  my $self = shift;
  my $companyObj = $self->{companyObj};
  my $userObj = $self->{userObj};
  my $appObj = $self->{appObj};
  my $errStr = "Portal::squidGuard->cleanupUser()  - Error!<br>\n";

  # do any cleanup necessary
}

1;
__END__

=head1 NAME

PortalSetup - Object used to build a PortalSetup Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::PortalSetup;
  my $obj = Portal::squidGuard::PortalSetup->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

PortalSetup is a PortalSetup class.

=head1 Exported FUNCTIONS

  scalar new(langObj, input, cookieObj, dbHandle, logObj, sessionObj,
             userObj, companyObj, applicationObj, appObj, portalSession,
             browserType, browserVersion, browserMaker, authObj)
    Creates a new instance of the Portal::PortalSetup
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will 
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.
    
  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


  void cleanupDatabase(dataDir)
    This routine dumps any data necessary and if a database is defined,
    destroys the database.

  HTMLObject setupCompany()
    This routine proceeds to create a database for the company if needed
    and does any initialization necessary.  The application needs to
    provide an HTMLObject document that will start the user going through
    a configuration program for that application.  If the doc is undef on
    return and no error is set, then the calling app will assume that no
    extra configuration needs to be done.  The calling program should
    have this output being done in another window and provide the user
    with a link to close that window when all configuration is done.

  HTMLObject setupUser()
    This routine configures the user in the application.  The application
    needs to provide an HTMLObject document that will start the user going
    through a configuration program for that application.  If the doc is
    undef on return and no error is set, then the calling app will assume
    that no extra configuration needs to be done.  The calling program
    should have this output being done in another window and provide the
    user with a link to close that window when all configuration is done.

  HTMLObject updateUser()
    This routine updates the users settings in the application.  This is
    mainly to synchronize any changes that were made to the users info in
    the portal_db that the application duplicates or relies on for user
    settings in its own database.  The application needs to provide an
    HTMLObject document that will start the user going through a
    configuration program for that application.  If the doc is undef on
    return and no error is set, then the calling app will assume that no
    extra configuration needs to be done.  The calling program should have
    this output being done in another window and provide the user with a
    link to close that window when all configuration is done.

  void cleanupUser()
    This routine archives any info on the user and then removes their data
    from active use.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
